----------------------------------------------------------------------------------
-- Company:  FE, 2014
-- Engineer: A. Trost
-- 
-- Create Date: jan 2014
-- Design Name: VGAwin
-- Module Name: VGAwin - Behavioral
-- Project Name: VGAwin, Zynq delavnica 
-- Target Devices: xc7z020clg484-1
-- Tool Versions:  Vivado 2013.3
-- Description: VGA prikaz okna 512x512 iz 256k BRAM 
--  clk 100MHz, VGA clk: 50MHz, 800x600
-- 
-- Revision: 2.0
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity VGAwin is
    Port ( clk : in STD_LOGIC;
           rstn : in STD_LOGIC;
           --pixin : in STD_LOGIC_VECTOR (5 downto 0);
           --valid : in STD_LOGIC;
           --x, y : out STD_LOGIC_VECTOR(15 downto 0);
           --intr : out STD_LOGIC;
           BRAM_enb : out STD_LOGIC;           
           BRAM_doutb : out STD_LOGIC_VECTOR (31 downto 0);
           BRAM_dinb : in STD_LOGIC_VECTOR (31 downto 0);
           BRAM_web : out STD_LOGIC_VECTOR (3 downto 0);          
           BRAM_addrb : out STD_LOGIC_VECTOR (31 downto 0); 
           BRAM_clkb : out STD_LOGIC;
           BRAM_rstb : out STD_LOGIC;
           
           hsync : out  STD_LOGIC;
           vsync : out  STD_LOGIC;         
           rgb : out  STD_LOGIC_VECTOR (5 downto 0)
           );
end VGAwin;

architecture Behavioral of VGAwin is
 signal vgaclk: std_logic := '0';
 
 signal hst: unsigned(10 downto 0):=(others=>'0');
 signal vst: unsigned(9 downto 0):=(others=>'0');
 constant xc: unsigned(10 downto 0):="00010010000";
 constant yc: unsigned(9 downto 0):= "0000101100"; 
 signal xr: std_logic_vector(7 downto 0);
 signal yr: std_logic_vector(6 downto 0);
 signal rgbw: std_logic_vector(5 downto 0);
begin

-- delilnik ure
pclk: process(clk)
begin
 if rising_edge(clk) then
   vgaclk <= not vgaclk;     -- 50 MHz	
 end if;
end process;

-- koordinati trenutne tocke
--y <= std_logic_vector(resize(vst, 16)); 
--x <= std_logic_vector(resize(hst, 16));

-- relativni koordinati v oknu
xr <= std_logic_vector(hst(7 downto 0)-xc(7 downto 0));  
yr <= std_logic_vector(vst(6 downto 0)-yc(6 downto 0));

-- BRAM kontrolni signali, za test komentiraj vse signale
BRAM_addrb <= "00000000000000000" & yr & xr;
BRAM_enb <= '1';
BRAM_doutb <= (others => '0');
BRAM_web <= (others => '0');
BRAM_rstb <= not rstn;
BRAM_clkb <= clk;

-- VGA timing proces
pVGA: Process(clk)
begin
 if rising_edge(clk) then
   if vgaclk='1' then
	if hst < 1039 then 
		hst<=hst+1;
	else
		hst<=(others=>'0');
		if vst < 665 then
			vst<=vst+1;
		else
			vst<=(others=>'0');
		end if;
	end if;	
	
--	if hst=800 then  -- prekinitev ob koncu vidne slike
--	   intr <= '1';
--	else
--	   intr <= '0';
--	end if;
			
	if hst<800 and vst<600 then	 						
		if hst>=xc and hst<xc+256 and vst>=yc and vst<yc+128 then
		  rgb <= "111100";
			case xr(1 downto 0) is  -- za test komentiraj celoten stavek case
			 when "00" => rgb <= BRAM_dinb(5 downto 0);
			 when "01" => rgb <= BRAM_dinb(13 downto 8);
			 when "10" => rgb <= BRAM_dinb(21 downto 16);
			 when others => rgb <= BRAM_dinb(29 downto 24);
			end case;
		else
		  rgb <= "000001"; -- temno modra barva ozadja
		end if;	
		
--    if valid='1' then  -- risanje zunanje slike (pixin), 000000 je transparentna barva
--      if pixin/="000000" then
--        rgb <= pixin;
--      end if;
--    end if; 	
	else
		rgb<="000000"; -- zatemnitev
	end if;
  end if;	
 end if;
end process;

hsync <= '1' when hst>=856 and hst<976 else '0'; 
vsync <= '1' when vst>=637 and vst<643 else '0';   

end Behavioral;
